; ----------------------------------------------------------------------+
; FROTH - A forth language implementation on PC - Oliver SINGLA 	!
; 									!
; Module:	EDITOR.ASM						!
; 									!
; Description:	Definitions for playing with editor initializations.	!
; 									!
; List of FROTH definitions defined in this module:			!
;	EDC_SLL			C_StatusLabel				!
;	EDC_SLI			C_StatusInfos				!
;	EDC_TXN			C_NormalText				!
;	EDC_TXS			C_SelectedText				!
;	EDC_HLN			C_NormHelp				!
;	EDC_HLA			C_AltHelp				!
;	EDC_HLK			C_KeysHelp				!
;	ED_VIDEOMODE		VideoMode				!
;	ED_DIR			DIR$					!
;	ED_QDIRECTVIDEO		?DirectVideo				!
;	ED_QUSETAB		?UseTab					!
;	ED_COMMENTFIELD		CommenField				!
;	ED_LINEPOSADJUST	LinePosAdjust				!
;	ED_KEY			EditKey					!
; 									!
; ----------------------------------------------------------------------+

		.386p
		jumps
		locals

code32  	segment para public use32
		assume cs:code32, ds:code32, ss:code32

include		pmode.inc				; All externs are defined in there

include		Equ.inc
include		Vars.inc
include		Dict.inc

include		Ed.inc                      		; Constants, Structurs, Variables used by FROSTED


; ===============================================================
; = GiveDataAddr					    	=
; =								=
; = Usage:							=
; =	!!!							=
; =								=
; = E) nant                    	                    	=
; =								=
; = O) eax : value read						=
; =								=
; = REGISTERS ARE PRESERVED.					=
; ===============================================================

		align	4
GiveDataAddr	proc

		; Get data reference
		mov	eax, [esp]
		mov	eax, [eax]

		; Move the IP after the data reference
		add	dword ptr [esp], @WordSize

		; Push the data address
		add	eax, [DDIR_START]
		mov	eax, [eax.ddir_blkaddr]

		; Return to caller
		ret

GiveDataAddr	endp


; ===============================================================
; = C_StatusLabel                       			=
; =								=
; = Usage:							=
; =								=
; =                                                         	=
; = Stack:  ( --- adr )	                 			=
; ===============================================================

		align	4
EDC_SLL		proc

		; Is editor active ?
		cmp	[EditorActive], FALSE
		jne	@@EdOn

		; Integer run-time
		DOINT  	D_EDC_SLL
		OVSTACK					; Verify stack frame overflow, then return to caller	
		ret

		; Editor is active
@@EdOn:		call	GiveDataAddr
		dd  	D_EDC_SLL

		; Return to caller
@@Exit:		ret

E_EDC_SLL:
EDC_SLL		endp


; ===============================================================
; = C_StatusInfos                      				=
; =								=
; = Usage:							=
; =								=
; =                                                         	=
; = Stack:  ( --- adr )	                 			=
; ===============================================================

		align	4
EDC_SLI		proc

		; Is editor active ?
		cmp	[EditorActive], FALSE
		jne	@@EdOn

		; Integer run-time
		DOINT  	D_EDC_SLI
		OVSTACK					; Verify stack frame overflow, then return to caller	
		ret

		; Editor is active
@@EdOn:		call	GiveDataAddr
		dd  	D_EDC_SLI

		; Return to caller
@@Exit:		ret

E_EDC_SLI:
EDC_SLI		endp


; ===============================================================
; = C_NormalText                      				=
; =								=
; = Usage:							=
; =								=
; =                                                         	=
; = Stack:  ( --- addr )	         	        	=
; ===============================================================

		align	4
EDC_TXN		proc

		; Is editor active ?
		cmp	[EditorActive], FALSE
		jne	@@EdOn

		; Integer run-time
		DOINT  	D_EDC_TXN
		OVSTACK					; Verify stack frame overflow, then return to caller	
		ret
		
		; Editor is active
@@EdOn:		call	GiveDataAddr
		dd  	D_EDC_TXN

		; Return to caller
@@Exit:		ret

E_EDC_TXN:
EDC_TXN		endp


; ===============================================================
; = C_SelectedText                      			=
; =								=
; = Usage:							=
; =	Color for: selected text				=
; =								=
; = Stack:  ( --- addr )	         	        	=
; ===============================================================

		align	4
EDC_TXS		proc

		; Is editor active ?
		cmp	[EditorActive], FALSE
		jne	@@EdOn

		; Integer run-time
		DOINT  	D_EDC_TXS
		OVSTACK					; Verify stack frame overflow, then return to caller	
		ret

		; Editor is active
@@EdOn:		call	GiveDataAddr
		dd  	D_EDC_TXS

		; Return to caller
@@Exit:		ret

E_EDC_TXS:
EDC_TXS		endp


; ===============================================================
; = C_NormHelp                      				=
; =								=
; = Usage:							=
; =	Color for: normal, help					=
; =								=
; = Stack:  ( --- addr )	                 		=
; ===============================================================

		align	4
EDC_HLN		proc

		; Is editor active ?
		cmp	[EditorActive], FALSE
		jne	@@EdOn

		; Integer run-time
		DOINT  	D_EDC_HLN
		OVSTACK					; Verify stack frame overflow, then return to caller	
		ret

		; Editor is active
@@EdOn:		call	GiveDataAddr
		dd  	D_EDC_HLN

		; Return to caller
@@Exit:		ret

E_EDC_HLN:
EDC_HLN		endp


; ===============================================================
; = C_AltHelp                      				=
; =								=
; = Usage:							=
; =	Color for: alt, help					=
; =								=
; = Stack:  ( --- addr )	         	        	=
; ===============================================================

		align	4
EDC_HLA		proc

		; Is editor active ?
		cmp	[EditorActive], FALSE
		jne	@@EdOn

		; Integer run-time
		DOINT  	D_EDC_HLA
		OVSTACK					; Verify stack frame overflow, then return to caller	
		ret

		; Editor is active
@@EdOn:		call	GiveDataAddr
		dd  	D_EDC_HLA

		; Return to caller
@@Exit:		ret

E_EDC_HLA:
EDC_HLA		endp


; ===============================================================
; = C_KeysHelp                      				=
; =								=
; = Usage:							=
; =	Color for: keys, help					=
; =								=
; = Stack:  ( --- addr )	                 		=
; ===============================================================

		align	4
EDC_HLK		proc

		; Is editor active ?
		cmp	[EditorActive], FALSE
		jne	@@EdOn

		; Integer run-time
		DOINT  	D_EDC_HLK
		OVSTACK					; Verify stack frame overflow, then return to caller	
		ret

		; Editor is active
@@EdOn:		call	GiveDataAddr
		dd  	D_EDC_HLK

		; Return to caller
@@Exit:		ret

E_EDC_HLK:
EDC_HLK		endp


; ===============================================================
; = ED_VIDEOMODE						=
; =								=
; = Usage:							=
; =	Setting video mode.					=
; =								=
; = Stack:  ( --- adr )	                 			=
; ===============================================================

		align	4
ED_VIDEOMODE	proc

		; Is editor active ?
		cmp	[EditorActive], FALSE
		jne	@@EdOn

		; Integer run-time
		DOINT  	D_ED_VIDEOMODE
		OVSTACK					; Verify stack frame overflow, then return to caller	
		ret

		; Editor is active
@@EdOn:		call	GiveDataAddr
		dd  	D_ED_VIDEOMODE

		; Return to caller
@@Exit:		ret

E_ED_VIDEOMODE:
ED_VIDEOMODE	endp


; ===============================================================
; = ED_DIR							=
; =								=
; = Usage:							=
; =	Setting editor files default directory.			=
; =								=
; = Stack:  ( --- addr )	                 		=
; ===============================================================

		align	4
ED_DIR		proc

		call	@@next
@@next:		add	dword ptr [esp], 4+6+3+2+1
		mov	eax, [edi.ddir_blkaddr + D_ED_DIR]
		IPUSH	eax
		ret
		
		; 4+4+2+2
		add	PARAM0, size strentry
		
		; Return to caller
@@Exit:		ret

E_ED_DIR:
ED_DIR		endp


; ===============================================================
; = ED_QDIRECTVIDEO						=
; =								=
; = Usage:							=
; =	Do we use direct video functions  ?			=
; =								=
; = Stack:  ( --- adr )	                 			=
; ===============================================================

		align	4
ED_QDIRECTVIDEO	proc

		; Is editor active ?
		cmp	[EditorActive], FALSE
		jne	@@EdOn

		; Integer run-time
		DOINT  	D_ED_QDIRECTVIDEO
		OVSTACK					; Verify stack frame overflow, then return to caller	
		ret

		; Editor is active
@@EdOn:		call	GiveDataAddr
		dd  	D_ED_QDIRECTVIDEO

		; Return to caller
@@Exit:		ret

E_ED_QDIRECTVIDEO:
ED_QDIRECTVIDEO	endp


; ===============================================================
; = ED_QUSETAB							=
; =								=
; = Usage:							=
; =	Do we use direct video functions  ?			=
; =								=
; = Stack:  ( --- adr )	                 			=
; ===============================================================

		align	4
ED_QUSETAB	proc

		; Is editor active ?
		cmp	[EditorActive], FALSE
		jne	@@EdOn

		; Integer run-time
		DOINT  	D_ED_QUSETAB
		OVSTACK					; Verify stack frame overflow, then return to caller	
		ret

		; Editor is active
@@EdOn:		call	GiveDataAddr
		dd  	D_ED_QUSETAB

		; Return to caller
@@Exit:		ret

E_ED_QUSETAB:
ED_QUSETAB	endp


; ===============================================================
; = ED_COMMENTFIELD						=
; =								=
; = Usage:							=
; =	Column position of comment field.			=
; =								=
; = Stack:  ( --- addr )	                 		=
; ===============================================================

		align	4
ED_COMMENTFIELD	proc

		; Is editor active ?
		cmp	[EditorActive], FALSE
		jne	@@EdOn

		; Integer run-time
		DOINT  	D_ED_COMMENTFIELD
		OVSTACK					; Verify stack frame overflow, then return to caller	
		ret

		; Editor is active
@@EdOn:		call	GiveDataAddr
		dd  	D_ED_COMMENTFIELD

		; Return to caller
@@Exit:		ret

E_ED_COMMENTFIELD:
ED_COMMENTFIELD	endp


; ===============================================================
; = ED_LINEPOSADJUST						=
; =								=
; = Usage:							=
; =	Nb of lines scrolled down when goto a given line.	=
; =								=
; = Stack:  ( --- addr )	                 		=
; ===============================================================

		align	4
ED_LINEPOSADJUST	proc

		; Is editor active ?
		cmp	[EditorActive], FALSE
		jne	@@EdOn

		; Integer run-time
		DOINT  	D_ED_LINEPOSADJUST
		OVSTACK					; Verify stack frame overflow, then return to caller	
		ret

		; Editor is active
@@EdOn:		call	GiveDataAddr
		dd  	D_ED_LINEPOSADJUST

		; Return to caller
@@Exit:		ret

E_ED_LINEPOSADJUST:
ED_LINEPOSADJUST	endp


; ===============================================================
; = ED_KEY							=
; =								=
; = Usage:							=
; =	The key which calls FROSTED under INTERPRET.		=
; =								=
; = Stack:  ( --- addr )	                 		=
; ===============================================================

		align	4
ED_KEY		proc

		; Integer run-time
		DOINT  	D_ED_KEY

		; Verify stack frame overflow, then return to caller
		OVSTACK
		
		; Return to caller
		ret

E_ED_KEY:
ED_KEY		endp


code32		ends
		end
