; ----------------------------------------------------------------------+
; FROTH - A forth language implementation on PC - Oliver SINGLA 	!
; 									!
; Module:	ED_CMD5.ASM						!
; 									!
; Description:	Definitions for playing with FROSTED, built-in editor.	!
; 									!
; ----------------------------------------------------------------------+

		.386p
		jumps
		locals

code32 		segment para public use32
		assume cs:code32, ds:code32, ss:code32

include		pmode.inc				; All externs are defined in there

include		Equ.inc
include		Vars.inc
include		Dict.inc

include		Ed.inc					; Constants, Structurs, Variables used by FROSTED


; ===============================================================
; = EdRecordMacro						=
; =								=
; = Called by 'IsKeyACommand' : 				=
; = 	Record a new macro.					=
; =								=
; = I)	none							=
; =								=
; = O)	none							=
; =								=
; = M)	all registers.						=
; ===============================================================

		align	4
EdRecordMacro	proc

		; ------------------------
		; Print msg on status line
		; ------------------------

		lea	edx, EdMsgRecordMac		
		call	EdPrMsgStatus


		; ---------------
		; Wait 'A' to 'Z'
		; ---------------

		clr	ah					; Row #
		mov	al, 43					; Col #
		call	EdGotoXY
@@WaitAZ:	call	CharIn
		cmp	ax, K_ESC
		je	@@Exit
		cmp	ax, 'a'
		jb	@@WaitAZ2
		cmp	ax, 'z'
		ja	@@WaitAZ2
		sub	ax, 20h
@@WaitAZ2:	cmp	ax, 'A'
		jb	@@WaitAZ
		cmp	ax, 'Z'
		ja	@@WaitAZ
		
		
		; ----------------
		; Return to caller
		; ----------------

@@Exit:
		
		; Redraw the entire status line
		call	EdPrStatusLine
		
		; Update the cursor position on screen
		call	EdPosCur

		; Return
		ret

EdRecordMacro	endp


; ===============================================================
; = EdInsFileHeader						=
; =								=
; = Called by 'IsKeyACommand' : 				=
; = 	Insert a file header.					=
; =								=
; = I)	none							=
; =								=
; = O)	none							=
; =								=
; = M)	all registers.						=
; ===============================================================

		align	4
EdInsFileHeader	proc

		; ----------------------------------
		; Cancel the block selection, if any
		; ----------------------------------
		
		call	EdInactiveBlock


		; -------------------------------
		; Put current line in edited text
		; -------------------------------

		call	EdPutCurLinBuff


		; ------------------------------------
		; Is the file-header already present ?
		; ------------------------------------

		; Nb of lines ?
		cmp	[ebp.binf_LinesNb], 9
		jl	@@NotPresent
		
		; 1st line seems to be the header ?
		mov	eax, [ebp.binf_BuffAddr]		
		cmp	dword ptr [eax], '- //'
		jne	@@NotPresent
		cmp	dword ptr [eax + 4], '----'
		je	@@Exit
		
@@NotPresent:


		; ------------------------
		; Generate the file header
		; ------------------------
		
		; Choose 80 or 132 col
		cmp	[EdColsNb], 80
		jne	@@132
		lea	edx, @@Header_80
		mov	ebx, @@End80 - @@Header_80
		jmp	@@80or132
@@132:		lea	edx, @@Header_132
		mov	ebx, @@End132 - @@Header_132
@@80or132:

		; ----------------------
		; Room for these bytes ?
		; ----------------------
		
		mov	eax, ebx
		call	EdRequestMem
		jc	@@Exit


		; --------------------------
		; Update the number of lines
		; --------------------------
		
		add	[ebp.binf_LinesNb], 11


		; --------------------------------------------------
		; Move the text, make a 'hole' where header will fit
		; --------------------------------------------------
		
		; The nb of bytes to move
		call	EdCompTextSize					; Size of the text into EAX
		mov	ecx, eax

		; Source address
		mov	esi, [ebp.binf_BuffAddr]
		add	esi, eax
		dec	esi						; ESI : last char of the text
		
		; Destination address
		mov	edi, esi
		add	edi, ebx
		
		; Move block
		std							; Dec ESI and EDI
		rep	movsb  			                	; Move/Copy the block
		cld
		
		
		; ------------------------------------
		; Now insert the header into the 'hole'
		; ------------------------------------

		; Source address
		mov	esi, edx
		
		; Destination address
		mov	edi, [ebp.binf_BuffAddr]
		
		; The nb of bytes to move
		mov	ecx, ebx

		; Move block
		cld
		rep	movsb


		; -------------------------------------------------
		; Update the position of tagged marks / definitions
		; -------------------------------------------------

		mov	edi, 11						; Offset in nb of lines		
		call	EdUpdatePosTags


		; ------------
		; Set OVR mode
		; ------------
		
		mov	[EdInsertMode], FALSE


		; --------------------------
		; Current buffer is modified
		; --------------------------

		call	EdCurrBuffIsMod


		; ----------------
		; Goto top of text
		; ----------------
		
		jmp	EdDoMoveBeginTxt


		; ----------------
		; Return to caller
		; ----------------
		
@@Exit:		

		; Redraw the entire status line
		call	EdPrStatusLine
		
		; Update the cursor position on screen
		call	EdPosCur

		; Return
		ret

		align	4		
@@Header_80:	db	"// -------------------------------------------------------------------------- \\", 13, 10
		db	"//                                                                            \\", 13, 10
		db	"// File:               Author:                Date:           Version:        \\", 13, 10
		db	"// ----                ------                 ----            -------         \\", 13, 10
		db	"//                                                                            \\", 13, 10
		db	"// Description:                                                               \\", 13, 10
		db	"//                                                                            \\", 13, 10
		db	"//                                                                            \\", 13, 10
		db	"// -------------------------------------------------------------------------- \\", 13, 10
		db	13, 10
		db	13, 10
@@End80:

		align	4		
@@Header_132:	db	"// ---------------------------------------------------------------------------------------------------------------------------- \\", 13, 10
		db	"//                                                                                                                              \\", 13, 10
		db	"// File:                Author:                  Date:              Version:                                                    \\", 13, 10
		db	"// ----                 ------                   ----               -------                                                     \\", 13, 10
		db	"//                                                                                                                              \\", 13, 10
		db	"// Description:                                                                                                                 \\", 13, 10
		db	"//                                                                                                                              \\", 13, 10
		db	"//                                                                                                                              \\", 13, 10
		db	"// ---------------------------------------------------------------------------------------------------------------------------- \\", 13, 10
		db	13, 10
		db	13, 10
@@End132:


EdInsFileHeader	endp


; ===============================================================
; = EdInsWordHeader						=
; =								=
; = Called by 'IsKeyACommand' : 				=
; = 	Insert a word header.					=
; =								=
; = I)	none							=
; =								=
; = O)	none							=
; =								=
; = M)	all registers.						=
; ===============================================================

		align	4
EdInsWordHeader	proc

		; -------------------------------
		; Put current line in edited text
		; -------------------------------

		call	EdPutCurLinBuff


		; ----------------------------------
		; Cancel the block selection, if any
		; ----------------------------------
		
		call	EdInactiveBlock


		; ------------------------
		; Generate the file header
		; ------------------------
		
		; Choose 80 or 132 col
		cmp	[EdColsNb], 80
		jne	@@132
		lea	edx, @@Header_80
		mov	ebx, @@End80 - @@Header_80
		jmp	@@80or132
@@132:		lea	edx, @@Header_132
		mov	ebx, @@End132 - @@Header_132
@@80or132:

		; -----------------
		; Ask for new space
		; -----------------
		mov	eax, ebx
		call	EdRequestMem
		jc	@@Exit


		; --------------------------
		; Update the number of lines
		; --------------------------
		
		add	[ebp.binf_LinesNb], 8


		; --------------------------------------------------
		; Move the text, make a 'hole' where header will fit
		; --------------------------------------------------
		
		; Source address
		call	EdCompTextSize					; Size of the text into EAX
		mov	esi, eax
		add	esi, [ebp.binf_BuffAddr]
		dec	esi						; ESI : last char of the text
		
		; Destination address
		mov	edi, esi
		add	edi, ebx
		
		; The nb of bytes to move
		mov	ecx, esi
		sub	ecx, [ebp.binf_CurLinAddr]
		inc	ecx

		; Move block
		std							; Dec ESI and EDI
		rep	movsb  			                	; Move/Copy the block
		cld
		
		
		; ------------------------------------
		; Now insert the header into the 'hole'
		; ------------------------------------

		; Source address
		mov	esi, edx
		
		; Destination address
		mov	edi, [ebp.binf_CurLinAddr]
		
		; The nb of bytes to move
		mov	ecx, ebx

		; Move block
		cld
		rep	movsb


		; --------------------------
		; Current buffer is modified
		; --------------------------
		
		call	EdCurrBuffIsMod


		; -------------------------------------------------
		; Update the position of tagged marks / definitions
		; -------------------------------------------------

		mov	edi, 8						; Offset in nb of lines		
		call	EdUpdatePosTags


		; -----------
		; Redraw page
		; -----------
		
		call	EdRedrawPage


		; ----------------------
		; Update cursor position
		; ----------------------

		; Column
		mov	[ebp.binf_fcol], 0
		mov	[ebp.binf_tcol], 17
		
		; Next row, and bring the new line into editing buffer
		inc	[ebp.binf_trow]						; Row++
		mov	ecx, [ebp.binf_CurLinAddr]
		call	EdGetCurLinBuff
		mov	[ebp.binf_CurLinAddr], ecx
		call	EdGetCurLinBuff
		
		; OVR
		mov	[EdInsertMode], FALSE


		; ----------------
		; Return to caller
		; ----------------
		
@@Exit:		

		; Redraw the entire status line
		call	EdPrStatusLine
		
		; Update the cursor position on screen
		call	EdPosCur

		; Return
		ret
		
		align	4		
@@Header_80:	db	"// -------------------------------------------------------------------------- \\", 13, 10
		db	"//       Word :                                                              \\", 13, 10
		db	"//      Stack :                                                              \\", 13, 10
		db	"// Description :                                                              \\", 13, 10
		db	"//                                                                           \\", 13, 10
		db	"//    Callers :                                                              \\", 13, 10
		db	"//      Calls :                                                              \\", 13, 10
		db	"// -------------------------------------------------------------------------- \\", 13, 10
@@End80:

		align	4		
@@Header_132:	db	"// ---------------------------------------------------------------------------------------------------------------------------- \\", 13, 10
		db	"//       Word :                                                                                                                \\", 13, 10
		db	"//      Stack :                                                                                                                \\", 13, 10
		db	"// Description :                                                                                                                \\", 13, 10
		db	"//                                                                                                                             \\", 13, 10
		db	"//    Callers :                                                                                                                \\", 13, 10
		db	"//      Calls :                                                                                                                \\", 13, 10
		db	"// ---------------------------------------------------------------------------------------------------------------------------- \\", 13, 10
@@End132:

EdInsWordHeader	endp


code32		ends
		end
