; ----------------------------------------------------------------------+
; FROTH - A forth language implementation on PC - Oliver SINGLA 	!
; 									!
; Module:	HPIXEL.ASM						!
; 									!
; Description:	Definitions for playing with graphics.			!
; 		(drawing pixel).					!
; 									!
; List of FROTH definitions defined in this module:			!
;	SETPIXEL	SetPixel					!
;	GETPIXEL	GetPixel					!
; 									!
; ----------------------------------------------------------------------+

		.386p
		jumps
		locals

code32 		segment para public use32
		assume cs:code32, ds:code32, ss:code32

include		pmode.inc				; All externs are defined in there

include		Equ.inc
include		Vars.inc
include		Dict.inc

include		hgr.inc



; InitVNWait
;
;   void VS2API InitVNWait( void );
;
; Notes:
;   Initializes a timeout value for detecting the start of the
;   vertical nondisplay interval.

INITVNWAIT	proc

		push	ax cx dx

		push	edi gs
		mov	gs, [_selzero]
		mov	edi, (segVDDA shl 4)		; EDI -> BIOS data area
		mov	dx, gs:[edi + ADDR6845]
		pop	gs edi
		add	dx, 6				; DX = Status Register 1 I/O addr

		; wait for vertical retrace
L101:		in	al, dx
		test	al, 8
		jz	L101

		; initialize the loop counter
		mov	cx, 0FFFFh			; CX = FFFFh

		; wait for display enable
L102:		in	al, dx
		test	al, 1
		jnz	L102

		; wait for the end of display enable
		cli					; disable interrupts
L103:		in	al, dx
		test	al, 1
		jz	L103

		; loop until display enable becomes active
L104:		in	al, dx
		test	al, 1
		loopwnz	L104
		sti					; enable interrupts

		neg	cx				; make CX positive
		add	cx, cx				; double it for safety
		mov	_VNLoopMax, cx			; save the result

		pop	dx cx ax
		ret

INITVNWAIT	endp



; VNWait
;
;   void VS2API VNWait( void );
;
; Notes:
;   Waits for "display enable end" by timing out when bit 0
;   of VGA Input Status Register 1 stays set to 1.  This indicates
;   that horizontal display enable is remaining inactive, that is,
;   that the vertical nondisplay interval has begun.
;
;   Call InitVNWait prior to using this function.

		align	4
VNWAIT		proc

		push	ax cx dx

		push	edi gs
		mov	gs, [_selzero]
		mov	edi, (segVDDA shl 4)			; EDI -> BIOS data area
		mov	dx, gs:[edi + ADDR6845]
		pop	gs edi
		add	dx, 6				; DX = Status Register 1 I/O addr
		
		; wait for display enable
L201:		in	al, dx
		test	al, 1
		jnz	L201

		; wait for the end of display enable
L202:		mov	cx, _VNLoopMax				; CX = maximum number of loops
L203:		in	al, dx
		test	al, 1
		jz	L203

		; wait for display enable
L204:		in	al, dx
		test	al, 1
		loopwnz	L204
		jz	L202					; jump if display enable detected
	
		pop	dx cx ax
		ret

		align	4
_VNLoopMax	dw	?

VNWAIT		endp


	; parameters for all PixelAddr functions
	; --------------------------------------
	
pix_Y	=	[ebp + 8]
pix_X	=	[ebp + 10]

		align	4
PixelAddr_10	proc

	; Notes:
	;   Valid in VGA video modes that use planar pixel mapping:
	;	BIOS mode 0Dh:	320x200 16-color
	;	BIOS mode 0Eh:	640x200 16-color
	;	BIOS mode 0Fh:	640x350 monochrome (4-color)
	;	BIOS mode 10h:	640x350 16-color
	;	BIOS mode 11h:	640x480 2-color
	;	BIOS mode 12h:	640x480 16-color

		enter	0, 0

		mov	ax, pix_Y
		mul	VSP.vsp_cbRow			; AX = y * (bytes per row)

		mov	dx, pix_X
		shr	dx, 3				; DX = x/8
		add	ax, dx				; AX = y*(bytes per row) + x/8

		movzx	edi, ax
		add	edi, VSP.vsp_VBuf			; 0A0000h

		leave
		ret	2*2

PixelAddr_10	endp


		align	4
PixelAddr_13	proc

	; Notes:
	;   Valid in VGA BIOS mode 13h (320x200 256-color)

		enter	0, 0

		mov	ax, pix_Y
		mul	VSP.vsp_cbRow			; AX = y * (bytes per row)
		add	ax, pix_X			; AX = y*(bytes per row) + x

		movzx	edi, ax
		add	edi, VSP.vsp_VBuf			; 0A0000h

		leave
		ret	2*2

PixelAddr_13	endp


; Parameters for all SetPixel functions
; -------------------------------------

ucPixel	=	byte ptr [ebp + 8]
usY	=	word ptr [ebp + 10]
usX	=	word ptr [ebp + 12]


		align	4
SetPixel_10	proc

		enter	0, 0

		; point to the specified pixel
		mov	ax, usY
		mul	VSP.vsp_cbRow			; AX = y * (bytes per row)

		mov	dx, usX
		mov	cx, dx				; CX = x
		shr	dx, 3				; DX = x/8
		add	ax, dx				; AX = y*(bytes per row) + x/8

		movzx	ebx, ax				
		add	ebx, VSP.vsp_VBuf 		; ES:BX -> pixel

		; compute a mask for the pixel's bit position
		and	cl, 7				; CL = # of bits to shift
		mov	ax, 8008h			; AH = 80h (bit mask)
							; AL = 8 (Bit Mask reg #)
		shr	ah, cl				; AH = bit mask for pixel

		; set the Graphics Controller Bit Mask register
		mov	dx, 3CEh				; GC I/O address
		out	dx, ax

		; set the Graphics Controller Mode register
		mov	ax, 205h				; AL = Mode register #
							; AH = Write Mode 2 (bits 0,1)
							;      Read Mode 0 (bit 3)
		out	dx, ax

		; set the Data Rotate/Function Select register
		mov	ah, VSP.vsp_cRMW			; AH = Read-Modify-Write bits
		mov	al, 3				; AL = Data Rotate/Function Select
		out	dx, ax

		; set the pixel value
		mov	al, ucPixel			; AL = pixel value
		xchg	fs:[ebx], al			; load latches during CPU read;
							; update latches and bit planes
							;  during CPU write

		; restore default Graphics Controller registers
		mov	ax, 0FF08h			; Bit Mask
		out	dx, ax

		mov	ax, 5				; Mode register
		out	dx, ax

		mov	ax, 3				; Function Select
		out	dx, ax

		leave
		ret	3*2
		
SetPixel_10	endp


		align	4
SetPixel_13	proc

		enter	0, 0

		; point to the specified pixel
		mov	ax, usY
		mul	VSP.vsp_cbRow			; AX = y * (bytes per row)
		add	ax, usX				; AX = y*(bytes per row) + x

		movzx	ebx, ax
		add	ebx, VSP.vsp_VBuf			; ES:BX -> pixel

		mov	al, ucPixel			; AL = pixel value

		; select a raster op
		mov	ah, VSP.vsp_cRMW			; AH = Read-Modify-Write bits
		mov	al, 3				; AL = Data Rotate/Function Select
		mov	dx, 3CEh				; Graphics Controller I/O addr
		out	dx, ax

		; update the pixel value
		mov	al, ucPixel
		xchg	fs:[ebx], al

		; restore the Graphics Controller Data Rotate/Function Select register
		mov	ax, 3
		out	dx, ax

		leave
		ret	3*2

SetPixel_13	endp


		align	4
SetPixel_256	proc

		enter	0, 0

		imul	eax, 640
		
		mov	dl, bl
		add	ebx, eax

		and	dl, 3
		mov	ah, 1
		xchg	cl, dl
		shl	ah, cl
		xchg	cl, dl
		
		mov	dx, 03c4h
		mov	al, 2
		out	dx, ax
		
		lea	ebx, [0A0000h + ebx]
		sub	ebx, _code32a
		mov	[ebx], cl
ifdef NEVER
		; point to the specified pixel
		mov	cx, usX
		mov	bx, cx				; BX = x coordinate

		mov	ax, VSP.vsp_cbRow
		mul	usY				; AX = row offset
		shr	bx, 2				; BX = column offset
		add	bx, ax				; BX = buffer offset
movzx	ebx, bx
		add	ebx, VSP.vsp_VBuf			; ES:BX -> pixel

		; select the plane that contains the specified pixel
		and	cl, 3
		mov	ax, 0102h			; AL = Map Mask reg #
		shl	ah, cl				; AH = bit plane #

		mov	dx, 3C4h				; Sequencer
		out	dx, ax

		; select a raster op
		mov	ah, VSP.vsp_cRMW			; AH = Read-Modify-Write bits
		mov	al, 3				; AL = Data Rotate/Function Select
		mov	dl, 0CEh				; DX = 3CEh (Graphics Controller)
		out	dx, ax

		; update the pixel value
		mov	al, ucPixel
		xchg	fs:[ebx], al

		; restore the Graphics Controller Data Rotate/Function Select register
		mov	ax, 3
		out	dx, ax

		; restore the Sequencer Map Mask register
		mov	ax, 0F02h
		mov	dl, 0C4h
		out	dx, ax
endif

		leave
		ret	3*2
	
SetPixel_256	endp


; ===============================================================
; = SETPIXEL							=
; =								=
; = Code for the froth word:	SetPixel			=
; =								=
; = Stack:  ( x y o --- )					=
; ===============================================================

		align	4
SETPIXEL   	proc

		; ------------------
		; Verify stack frame
		; ------------------
		
		VSTACK	3


		; -------------------------------
		; Get parameters and do operation
		; -------------------------------

		push	fs
		mov	fs, [_selzero]
		mov	eax, [PARAM2]			; Get x
		mov	ebx, [PARAM1]			; Get y
		mov	ecx, [PARAM0]			; Get color
		DISCARD	3
		push	ax bx cx
		call	[_setpix_]
		pop	fs
		
		
		; ----------------
		; Return to caller
		; ----------------

		ret
			
E_SETPIXEL:
SETPIXEL		endp



usY	=	word ptr [ebp + 8]
usX	=	word ptr [ebp + 10]

		align	4
GetPixel_10	proc

		enter	0, 0
		push	edi

		; point to the specified pixel
		mov	ax, usY
		mul	VSP.vsp_cbRow			; AX = y * (bytes per row)

		mov	dx, usX
		mov	cx, dx				; CX = x
		shr	dx, 3				; DX = x/8
		add	ax, dx				; AX = y*(bytes per row) + x/8

		movzx	edi, ax
		add	edi, [VSP.vsp_VBuf]		; fs:[edi] -> offset

		; compute a mask for the pixel's bit position
		not	cl
		and	cl, 7				; CL = # of bits to shift
		mov	ch, 1
		shl	ch, cl				; CH = bit mask

		; read one byte from each bit plane
		mov	dx, 3CEh				; GC I/O address
		mov	ax, 304h				; AH = 3 (bit plane number)
							; AL = 4 (Read Map Select reg #)

		clr	bl				; BL = initial pixel value

L001:		out	dx, ax				; select a bit plane
		mov	bh, fs:[edi]			; read one byte
		and	bh, ch				; mask the desired bit
		neg	bh				; BH bit 7 = masked bit
		rol	bx, 1				; accumulate bits in BL
		dec	ah				; decrement bit plane number
		jge	L001

		movzx	eax, bl				; EAX = pixel value

		pop	edi
		leave
		ret	2*2

GetPixel_10	endp


		align	4
GetPixel_13	proc

		enter	0, 0

		; point to the specified pixel
		mov	ax, usY
		mul	VSP.vsp_cbRow			; AX = y * (bytes per row)
		add	ax, usX 				; AX = y*(bytes per row) + x

		cwde					; movzx	eax, ax
		add	eax, VSP.vsp_VBuf			; fs:[edx] -> pixel

		; get the pixel value
		movzx	eax, byte ptr fs:[eax]		; EAX = pixel value

		leave
		ret	2*2

GetPixel_13	endp


; ===============================================================
; = GETPIXEL							=
; =								=
; = Code for the froth word:	GetPixel			=
; =								=
; = Stack:  ( x y --- o )					=
; ===============================================================

		align	4
GETPIXEL   	proc

		; ------------------
		; Verify stack frame
		; ------------------
		
		VSTACK	2
		
		
		; -------------------------------
		; Get parameters and do operation
		; -------------------------------

		push	fs
		mov	fs, [_selzero]
		mov	eax, [PARAM1]			; Get x
		mov	ebx, [PARAM0]			; Get y
		DISCARD	1
		push	ax bx
		call	[_getpix_]
		mov	[PARAM0], eax
		pop	fs
		
		
		; ----------------
		; Return to caller
		; ----------------

		ret
			
E_GETPIXEL:
GETPIXEL	endp


code32		ends
		end
