; ----------------------------------------------------------------------+
; FROTH - A forth language implementation on PC - Oliver SINGLA 	!
; 									!
; Module:	INFIX.ASM						!
; 									!
; Description:	Definitions for playing with infixed expr evaluation.	!
; 									!
; List of FROTH definitions defined in this module:			!
;	OPENBRACE	{						!
;	CLOSEBRACE	}						!
; 									!
; ----------------------------------------------------------------------+

		.386p
		jumps
		locals

code32 		segment para public use32
		assume cs:code32, ds:code32, ss:code32

include		pmode.inc				; All externs are defined in there

include		Equ.inc
include		Vars.inc
include		Dict.inc

include		Compile.inc				; FROTH compiled definitions


; ------> LOCAL VARIABLES to this module -----------------------------------------------------------------------------------------

INFIXSTACK_MAXSZ	equ	8			; Max size of stack

		align	4
ParenthLevel	db	?				; Parenthesis level

		align	4
IsOperand	db	?				; Next word interpret as operand ? If not, operator.

		align	4
OperatorsNb	dd	?				; Operators stack: nb of data in stack
operator	struc
	operator_ptr		dd	?		; Pointer to word
	operator_methptr	dd	?		; Pointer to method, 0 if not a method
	operator_plevel		db	?		; Parenthesis level
operator	ends
OperatorsStack	operator INFIXSTACK_MAXSZ dup (<>)	; Stack for operators

; <------ end of local variables to this module ----------------------------------------------------------------------------------


; ===============================================================
; = OPENBRACE							=
; =								=
; = Code for the forth word:	{				=
; =								=
; = Stack:		( --- )					=
; ===============================================================

		align	4
OPENBRACE	proc

		; Verify if the system is in compile state
		call	QCOMPILE

		; Set the system in mode: Infixed Expression Evaluation
		mov	[InfixExprEvalQ], TRUE

		; Internal initializations
		mov	[ParenthLevel], 0
		mov	[OperatorsNb], 0			; Operators stack: nb of data in stack
		mov	[IsOperand], TRUE			; Next word interpreted as operand ? If not, operator.

		; Open a (
		call	BraceOpenParenth

		; Return to caller
		ret

E_OPENBRACE:
OPENBRACE	endp


; ===============================================================
; = CLOSEBRACE							=
; =								=
; = Code for the forth word:	}				=
; =								=
; = Stack:		( --- )					=
; ===============================================================

		align	4
CLOSEBRACE	proc

		; Close the last )
		call	BraceCloseParenth

		; Operators stack not empty ?
		cmp	[OperatorsNb], 0
		je	@@OK

		; "Infixed expression evaluation: need right parenthesis."
@@NOK:		mov	ah, ERR_BRACENEEDRIGHTPAR			; Error #
		clr	edx					; No additionnal msg
		clr	ecx					; No line #
		call	doError
		
		; Reset the system in mode: not Infixed Expression Evaluation
@@OK:		mov	[InfixExprEvalQ], FALSE

		; Return to caller
		ret

E_CLOSEBRACE:
CLOSEBRACE	endp



; ===============================================================
; = BraceOpenParenth						=
; =								=
; = Code for the forth word:	(				=
; =								=
; = Stack:		( --- )					=
; ===============================================================

		align	4
BraceOpenParenth	proc

		; Update the parenthesis level
		inc	[ParenthLevel]

		; Return to caller
		ret

BraceOpenParenth	endp


; ===============================================================
; = _PopOperator						=
; =								=
; = Usage:							=
; =    Pop one operator from operators stack. Called when a	=
; =    right ) was met, or end of expression.			=
; =								=
; = I) none.							=
; ===============================================================

		align	4
_PopOperator	proc

		cmp	[OperatorsNb], 0
		jne	@@OK

ret
		
		; "Infixed expression evaluation: syntax error."
		mov	ah, ERR_BRACESYNTAXERROR			; Error #
		clr	edx						; No additionnal msg
		clr	ecx						; No line #
		jmp	doError
		
@@OK:		dec	[OperatorsNb]
		mov	edx, [OperatorsNb]
		imul	edx, size operator
		CALL_rel32						; CALL rel32
		mov	eax, [OperatorsStack.operator_ptr + edx]
		sub	eax, [CDA_START]
		sub	eax, [CDA_SIZE]
		sub	eax, 4						; 4 for REL32
		call	CodeComma               			; Code,

		; Return to caller
		ret

_PopOperator	endp


; ===============================================================
; = BraceCloseParenth						=
; =								=
; = Code for the forth word:	)				=
; =								=
; = Stack:		( --- )					=
; ===============================================================

		align	4
BraceCloseParenth	proc

		; Try to close more pararenthesis than previously opened ?
		cmp	[ParenthLevel], 0
		jg	@@Ok

		; "Incorrect closing parenthesis."
		add	esp, @WordSize				; Discard a call level
		mov	ah, ERR_BRACERPARENTHBAD		; Error #
		clr	edx					; No additionnal msg
		clr	ecx					; No line #
		jmp	doError
		
@@Ok:		

		; Update the parenthesis level
		dec	[ParenthLevel]
		
		; Pop operator
		call	_PopOperator

		; Return to caller
		ret
		
BraceCloseParenth	endp


; ===============================================================
; = BraceAddOp							=
; =								=
; = Usage:							=
; =   Called by INTERPRET after parsing a word in the {...}	=
; =   mode: adding an operator or an operation.			=
; =								=
; = I) EDX : word code address, or number if CL==1		=
; =    CL  : flag: is EDX a number ?				=
; =    EBX : if 0 , not a method, else method code pointer	=
; ===============================================================

		align	4
BraceAddOp	proc

		; ----------------------------------------------------
		; Next word interpreted as operand ? If not, operator.
		; ----------------------------------------------------

		cmp	[IsOperand], FALSE
		je	@@Operator

		
		; ------------------
		; This is an operand
		; ------------------

		; Compile the operand: number or word
		cmpz	cl, FALSE
		jne	@@IsNb

		CALL_rel32					; CALL rel32
		mov	eax, edx
		sub	eax, [CDA_START]
		sub	eax, [CDA_SIZE]
		sub	eax, 4					; 4 for REL32
		call	CodeComma               		; Code,

		; Compile the pointer to method ?
		cmpz	ebx, 0
		je	@@Done
		CALL_rel32					; CALL rel32
		mov	eax, ebx
		sub	eax, [CDA_START]
		sub	eax, [CDA_SIZE]
		sub	eax, 4					; 4 for REL32
		call	CodeComma               		; Code,

		jmp	@@Done

		; This is a number to compile
@@IsNb:		mov	eax, edx
		SUB_ESI_i8	@WordSize			; Put 'SUB ESI, 4'
		MOV_xESIi8_i32					; Put 'MOV DWORD PTR [ESI], IMM32'
		call	CodeComma               		; Code,
@@Done:		

		; We expect now an operator
		mov	[IsOperand], FALSE

		jmp	@@exit
	

		; "Infixed expression evaluation: internal stack overflow."
@@StackOvflow:	mov	ah, ERR_BRACESTACKFULL			; Error #
		clr	edx					; No additionnal msg
		clr	ecx					; No line #
		jmp	doError
		

		; -------------------
		; This is an operator
		; -------------------

@@Operator:	mov	ebx, [OperatorsNb]
		cmp	bl, INFIXSTACK_MAXSZ
		je	@@StackOvflow

		; Error if it's a number
		cmpz	cl, FALSE
		je	@@NotANb

		; "Infixed expression evaluation: operator expected, number given."
		mov	ah, ERR_BRACEOPERATOREXPEC		; Error #
		clr	edx					; No additionnal msg
		clr	ecx					; No line #
		jmp	doError

@@NotANb:

		; If this operator is on the same parenthesis level as previous, pop previous
		cmpz	bl, 0
		je	@@DontPopPrev
		dec	ebx
		imul	ebx, size operator
		mov	al, [ParenthLevel]
		cmp	[OperatorsStack.operator_plevel + ebx], al
		jne	@@DontPopPrev
		push	edx
		call	_PopOperator
		pop	edx
		
@@DontPopPrev:	
		
		; Store into stack
		mov	eax, [OperatorsNb]
		imul	eax, size operator
		mov	[OperatorsStack.operator_ptr + eax], edx		; Pointer to word code
		mov	[OperatorsStack.operator_methptr + eax], ebx		; Pointer to method, 0 if not a method
		mov	bl, [ParenthLevel]
		mov	[OperatorsStack.operator_plevel + eax], bl
	
		; Update the nb of operators in the stack
		inc	[OperatorsNb]

		; We expect now an operand
		mov	[IsOperand], TRUE


		; ----------------
		; Return to caller
		; ----------------
		
@@exit:		ret

BraceAddOp	endp

code32		ends
		end
